# router/review_service.py
#
# from typing import List, Dict, Optional
#
# from src.reviewsense.mongo_db_config import get_db_connection
# from src.reviewsense.ratings_fetcher import RatingsFetcher
# from src.reviewsense.review_fetcher import ReviewFetcher
# from src.reviewsense.review_rater import ReviewRater
# from src.reviewsense.review_adder import ReviewAdder


# class ReviewService:
#     """Service class for handling product reviews"""
#
#     def __init__(self, collection_name: str = "android_phone_reviews",
#                  embedding_model: str = "sentence-transformers/all-mpnet-base-v2"):
#         """
#         Initialize the review service with vector database and embeddings
#         """
#         self.fetcher = ReviewFetcher()
#         self.rater = ReviewRater()
#         self.adder = ReviewAdder()
#         self.ratingsFetcher = RatingsFetcher()
#
#     def fetch_reviews(self, product_id: str, features: List[str], threshold: float = 0.5) -> Dict[str, List[str]]:
#         return self.fetcher.fetch_reviews(product_id, features, threshold)
#
#     def fetch_feature_ratings(self, reviews_by_feature: List[Dict[str, str]]) -> Dict[str, Dict[str, Optional[float]]]:
#         return self.rater.generate_feature_ratings(reviews_by_feature)
#
#     def add_review(self, product_id: str, review: str, rating: int) -> str:
#         return self.adder.add_review(product_id, review, rating)
#
#     def fetch_ratings(self, product_id):
#         return self.ratingsFetcher.fetch_ratings(product_id=product_id)


from typing import List, Dict, Optional

from src.reviewsense_ecom.mongo.mongo_db_config import get_db_connection
from src.reviewsense_ecom.service.ratings_fetcher import RatingsFetcher
from src.reviewsense_ecom.service.review_fetcher import ReviewFetcher
from src.reviewsense_ecom.service.review_rater import ReviewRater
from src.reviewsense_ecom.service.review_adder import ReviewAdder


class ReviewService:
    """Service class for handling product reviews"""

    def __init__(self, embedding_model="sentence-transformers/all-mpnet-base-v2"):


        """
        Initialize the review service with MongoDB connection.
        """

        # Vector-based processing components
        self.embedding_model = embedding_model
        self.fetcher = ReviewFetcher()
        self.rater = ReviewRater()
        self.adder = ReviewAdder()
        self.ratingsFetcher = RatingsFetcher()

    def _get_collection(self, collection_name):
        """Returns the requested MongoDB collection."""
        return get_db_connection(collection_name)

    # def get_review_by_feature(self, product_id, feature, collection_name="productReviews"):
    #     """
    #     Fetch reviews from MongoDB based on `product_id` and `feature`
    #     """
    #     collection = self._get_collection(collection_name)
    #
    #     query = {"product_id": str(product_id)}
    #     if feature.lower() == "overall":
    #         projection = {"_id": 0, "review": 1}
    #     else:
    #         query[f"features.{feature}"] = {"$exists": True}
    #         projection = {
    #             "_id": 0,
    #             f"features.{feature}.positive": 1,
    #             f"features.{feature}.negative": 1
    #         }
    #
    #     reviews = collection.find(query, projection)
    #
    #     result = []
    #     for review in reviews:
    #         if feature.lower() == "overall":
    #             result.append(review.get("review", ""))
    #         else:
    #             feature_data = review.get("features", {}).get(feature, {})
    #             result.extend(feature_data.get("positive", []))
    #             result.extend(feature_data.get("negative", []))
    #
    #     print(f"result of Afreedi API ----->>> {result}")
    #     return result
    #WORKING
    def get_review_by_feature(self, product_id, feature):
        """
        Fetch reviews from MongoDB based on `product_id` and `feature`
        """
        collection = self._get_collection("productReviews")

        query = {"product_id": str(product_id)}
        projection = {"_id": 0, "review": 1, "features": 1}

        reviews = collection.find(query, projection)
        p=list(collection.find(query, projection))

        result = []
        for review in reviews:
            if feature.lower() == "overall":
                result.append(review.get("review", ""))
            else:
                features = review.get("features", [])
                for feature_dict in features:
                    if feature in feature_dict:
                        feature_data = feature_dict[feature]
                        result.extend(feature_data.get("positive", []))
                        result.extend(feature_data.get("negative", []))

        logger.info(f"results------->>>>> {result}")
        return result

    def get_full_review_by_feature(self, product_id, feature):
        """
        Fetch reviews from MongoDB based on product_id and feature.
        If the feature is present, return the 'review' field.
        """
        collection = self._get_collection("productReviews")

        query = {"product_id": str(product_id)}
        projection = {"_id": 0, "review": 1, "features": 1}

        reviews = collection.find(query, projection)

        result = []
        for review in reviews:
            if feature.lower() == "overall":
                result.append(review.get("review", ""))
            else:
                features = review.get("features", [])
                for feature_dict in features:
                    if feature in feature_dict:
                        result.append(review.get("review", ""))
                        break

        return result

    def get_percentage_by_feature(self, product_id, feature, collection_name="productFeatureRater"):
        """
        Retrieves positive and negative review counts along with average rating for a given product feature.
        """
        collection = self._get_collection(collection_name)

        query = {
            "product_id": str(product_id),
            f"features.{feature}": {"$exists": True}
        }
        projection = {
            "_id": 0,
            f"features.{feature}.positive_count": 1,
            f"features.{feature}.negative_count": 1,
            f"features.{feature}.average_rating": 1
        }

        document = collection.find_one(query, projection)

        if not document:
            return {"message": "No reviews found for this product and feature."}

        feature_data = document.get("features", {}).get(feature, {})
        positive_count = feature_data.get("positive_count", 0)
        negative_count = feature_data.get("negative_count", 0)
        average_rating = feature_data.get("average_rating", None)

        sum_reviews = positive_count + negative_count
        positive_percentage = float((positive_count / sum_reviews) * 100) if sum_reviews else 0
        negative_percentage = float((negative_count / sum_reviews) * 100) if sum_reviews else 0

        return {
            "positive_percentage": f"{positive_percentage}%",
            "negative_percentage": f"{negative_percentage}%",
            "average_rating": average_rating
        }

    # 🔹 Fetch reviews using vector-based method
    def fetch_reviews(self, product_id: str, features: List[str], threshold: float = 0.5) -> Dict[str, List[str]]:
        return self.fetcher.fetch_reviews(product_id, features, threshold)

    # 🔹 Generate feature-specific ratings
    def generate_feature_ratings(self, reviews_by_feature: Dict[str, List[str]]) -> Dict[str, Optional[float]]:
        return self.rater.generate_feature_ratings(reviews_by_feature)

    def fetch_feature_ratings(self, reviews_by_feature: List[Dict[str, str]]) -> Dict[str, Dict[str, Optional[float]]]:
        return self.rater.generate_feature_ratings(reviews_by_feature)


    # 🔹 Add a new review
    def add_review(self, product_id: str, review: str, rating: int) -> str:
        return self.adder.add_review(product_id, review, rating)

    # 🔹 Fetch product ratings
    def fetch_ratings(self, product_id: str):
        return self.ratingsFetcher.fetch_ratings(product_id=product_id)
