from pydantic import BaseModel
from langchain.prompts import ChatPromptTemplate
from langchain_core.output_parsers import JsonOutputParser
from typing import List, Optional, Dict
from src.reviewsense_ecom.llm.llm import get_llm
import re


class FeatureReviews(BaseModel):
    feature_reviews: List[dict]  # Each review will include a sentence and sentiment


class FeatureExtractor:
    def __init__(self):
        self.llm = get_llm("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")
        self.parser = self._create_reviews_parser()
        self.prompt = self._create_extraction_prompt()

    def _create_reviews_parser(self) -> JsonOutputParser:
        """Create JSON parser for feature-specific reviews extraction"""
        return JsonOutputParser()

    def _create_extraction_prompt(self) -> ChatPromptTemplate:
        """Create prompt for extracting feature-specific reviews with enhanced rules and sentiment analysis."""
        template = """Extract sentences about the given feature from the list of reviews.

        Rules:
        - Extract only parts discussing the specific feature.
        - Remove unrelated parts connected by 'and' or 'but'.
        - Keep original wording and capitalization.
        - If there is only one review, apply the same rules to extract sentences about the feature.

        Reviews: {reviews}
        Feature: {feature}

        Return only the parts discussing the specific feature and perform sentiment analysis for each extracted sentence in this JSON format:
        {{
            "feature_reviews": [
                {{ 
                    "feature" : {feature}
                    "sentence": "relevant sentence 1",
                    "sentiment": "positive/negative/neutral"
                }},
                {{
                    "feature" : {feature}
                    "sentence": "relevant sentence 2",
                    "sentiment": "positive/negative/neutral"
                }}
            ]
        }}
        """
        return ChatPromptTemplate.from_template(template)

    # def extract_feature_reviews(self, reviews: List[str], feature: str) -> List[dict]:
    #     """
    #     Extract feature-specific sentences from reviews with sentiment analysis
    #
    #     Args:
    #         reviews: List of review texts
    #         feature: Target feature to extract
    #
    #     Returns:
    #         List[dict]: Feature-specific sentences with sentiment analysis
    #     """
    #     try:
    #         chain = self.prompt | self.llm | self.parser
    #         reviews_text = "\n".join(reviews)
    #
    #         result = chain.invoke({
    #             "reviews": reviews_text,
    #             "feature": feature
    #         })
    #
    #         parsed_data = FeatureReviews(**result)  # Validate and parse the result
    #         return parsed_data.feature_reviews
    #
    #     except Exception as e:
    #         print(f"Error extracting feature reviews: {e}")
    #         return []

    def extract_feature_reviews(self, review: str, features: List[str]) -> List[Dict[str, str]]:
        """
        Extract feature-specific sentences from reviews with sentiment analysis.

        Args:
            review (str): Review text.
            features (List[str]): List of features to extract.

        Returns:
            List[Dict[str, str]]: Feature-specific sentences with sentiment analysis.
        """
        try:
            extracted_reviews = []
            sentences = re.split(r'(?<=[.!?])\s+', review)  # Split review into sentences

            for feature in features:
                feature_sentences = [s for s in sentences if feature.lower() in s.lower()]

                for sentence in feature_sentences:
                    result = self.prompt | self.llm | self.parser
                    response = result.invoke({
                        "reviews": sentence,
                        "feature": feature
                    })

                    parsed_data = FeatureReviews(**response)  # Validate and parse result
                    extracted_reviews.extend(parsed_data.feature_reviews)
            print(f"Responce from LLM : {extracted_reviews}")

            return extracted_reviews

        except Exception as e:
            print(f"Error extracting feature reviews: {e}")
            return []
